package com.urbancode.air.plugin.udeploy.environments

import com.urbancode.air.AirPluginTool
import com.urbancode.ud.client.UDRestClient
import java.util.UUID
import java.net.URI

public class EnvironmentHelper {
    def apTool
    def props = []
    def udUser
    def udPass
    def weburl
    UDRestClient client
    
    public EnvironmentHelper(def apToolIn) {
        apTool = apToolIn
        props = apTool.getStepProperties()
        udUser = apTool.getAuthTokenUsername()
        udPass = apTool.getAuthToken()
        weburl = System.getenv("AH_WEB_URL")
        client = new UDRestClient(new URI(weburl), udUser, udPass)
        
        com.urbancode.air.XTrustProvider.install()
    }
    
    public def createEnvironment() {
        def appName = props['application']
        def envName = props['environment']
        def description = props['description']
        def requireApprovals = Boolean.valueOf(props['requireApprovals'])
        def appendSuffix = Boolean.valueOf(props['suffix'])
        if (!appName) {
            throw new IllegalArgumentException("no application was specified")
        }
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        if (!description) {
            description = ""
        }
        
        if (appendSuffix) {
            envName = envName + "-" + UUID.randomUUID().toString().substring(0,8)
        }
        
        UUID envUUID = client.createEnvironment(appName, envName, description, "white", requireApprovals)
        println "created environment with ID " + envUUID.toString()
        apTool.setOutputProperty("environment.name", envName)
        apTool.setOutputProperty("environment.id", envUUID.toString())
        apTool.setOutputProperties()
    }
    
    public def createComponentMapping() {
        def compName = props['component']
        def appName = props['application']
        def envNameOrId = props['environment']
        def resourceName = props['resource']
        String resourceGroupPath = props['resourceGroup']
        if (!compName) {
            throw new IllegalArgumentException("no component was specified")
        }
        if (!envNameOrId) {
            throw new IllegalArgumentException("no environment was specified")
        }
        
        if (resourceGroupPath) {
            //sanitize trailing slashes of resource group path
            while (resourceGroupPath.endsWith("/")) {
                resourceGroupPath = resourceGroupPath.substring(0, resourceGroupPath.length()-1);
            }
            if (resourceGroupPath.charAt(0) != '/') {
                resourceGroupPath = "/" + resourceGroupPath
            }
        }
        else {
            if (!resourceName) {
                throw new IllegalArgumentException("either a resource or resource group must be specified")
            }
            resourceGroupPath = ""
        }
        if (!resourceName) {
            resourceName = ""
        }
        
        UUID envUUID
        try {
            envUUID = UUID.fromString(envNameOrId);
        }
        catch (IllegalArgumentException e) {
            envUUID = client.getEnvironmentUUID(envNameOrId, appName);
        }
        client.createComponentMapping(envUUID, compName, resourceName, resourceGroupPath);
        println "component mapping created for environment " + envNameOrId + " and component " + compName
        apTool.setOutputProperty("environment.id", envUUID.toString())
        apTool.setOutputProperties()
    }
    
    public def setEnvironmentProperty() {
        def appName = props['application']
        def envName = props['environment']
        def propName = props['name']
        def propValue = props['value']
        def isSecure = Boolean.valueOf(props['isSecure'])
        
        if (!propName) {
            throw new IllegalArgumentException("no property name was specified")
        }
        if (!propValue) {
            propValue = ""
        }
        if (!appName) {
            appName = ""
        }
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        client.setEnvironmentProperty(envName, appName, propName, propValue, isSecure)
        println "environment property " + propName + " was set for environment " + envName
    }
    
    def setComponentEnvironmentProperty() {
        def compName = props['component']
        def appName = props['application']
        def envName = props['environment']
        def propName = props['name']
        def propValue = props['value']
        def isSecure = Boolean.valueOf(props['isSecure'])
        
        if (!propName) {
            throw new IllegalArgumentException("no property name was specified")
        }
        if (!propValue) {
            propValue = ""
        }
        if (!appName) {
            appName = ""
        }
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        if (!compName) {
            throw new IllegalArgumentException("no component was specified")
        }
        client.setComponentEnvironmentProperty(compName, envName, appName, propName, propValue, isSecure)
        println "component environment property " + propName + " was set for environment " + envName
    }
    
    def getEnvironmentProperties() {
        def appName = props['application']
        def envName = props['environment']
        
        if (!appName) {
            appName = ""
        }
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        
        Map<String, String> propMap = client.getEnvironmentProperties(envName, appName)
        Set<String> keySet = propMap.keySet()
        for(String key : keySet) {
            apTool.setOutputProperty(key, propMap.get(key))
        }
        apTool.setOutputProperties()
    }

    def getComponentEnvironmentProperties() {
        def appName = props['application']
        def envName = props['environment']
        def compName = props['component']
        
        if (!appName) {
            appName = ""
        }
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        if (!compName) {
            throw new IllegalArgumentException("no component was specified")
        }
        
        Map<String, String> propMap = client.getComponentEnvironmentProperties(compName, envName, appName)
        Set<String> keySet = propMap.keySet()
        for(String key : keySet) {
            apTool.setOutputProperty(key, propMap.get(key))
        }
        apTool.setOutputProperties()
    }
    
    def inactivateEnvironment() {
        def appName = props['application']
        def envName = props['environment']
        
        if (!appName) {
            appName = ""
        }
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        
        client.inactivateEnvironment(envName, appName);
        println "inactivated environment with name/id " + envName
    }
    
    def addUserToRole() {
        def user = props['user']
        def role = props['role']
        def envName = props['environment']
        def appName = props['application']
        
        if (!user) {
            throw new IllegalArgumentException("no user was specified")
        }
        if (!role) {
            throw new IllegalArgumentException("no role was specified")
        }
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        if (!appName) {
            throw new IllegalArgumentException("no application was specified")
        }
        client.addUserToRoleForEnvironment(user, role, envName, appName)
        println "added user ${user} to role ${role} for environment ${envName} in application ${appName}"
    }
    
    def addGroupToRole() {
        def group = props['group']
        def role = props['role']
        def envName = props['environment']
        def appName = props['application']
        
        if (!group) {
            throw new IllegalArgumentException("no group was specified")
        }
        if (!role) {
            throw new IllegalArgumentException("no role was specified")
        }
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        if (!appName) {
            throw new IllegalArgumentException("no application was specified")
        }
        client.addGroupToRoleForEnvironment(group, role, envName, appName)
        println "added user ${group} to role ${role} for environment ${envName} in application ${appName}"
    }
    
    def verifyInventoryStatus() {
        def envName = props['environment']
        def appName = props['application']
        def component = props['component']
        def version = props['version']
        def inventoryStatus = props['inventoryStatus']
        
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        if (!appName) {
            appName = ""
        }
        if (!component) {
            throw new IllegalArgumentException("no component was specified")
        }
        if (!version) {
            throw new IllegalArgumentException("no version was specified")
        }
        if (!inventoryStatus) {
            throw new IllegalArgumentException("no status was specified")
        }
        
        boolean verified = client.verifyInventoryStatus(envName, appName, component, version, inventoryStatus)
        if (verified) {
            println "Confirmed that environment ${envName} has component ${component} in inventory with version ${version} and status ${inventoryStatus}"
            apTool.setOutputProperty("verified", "true")
        }
        else {
            println "Environment ${envName} does NOT have component ${component} in inventory with version ${version} and status ${inventoryStatus}"
            apTool.setOutputProperty("verified", "false")
        }
        apTool.setOutputProperties()
    }
    
    def environmentExists() {
        def envName = props['environment']
        def appName = props['application']
        
        if (!envName) {
            throw new IllegalArgumentException("no environment was specified")
        }
        if (!appName) {
            appName = ""
        }
        
        try {
            def envID = client.getEnvironmentUUID(envName, appName)
            println "Environment with name ${envName} was found."
            apTool.setOutputProperty("exists", "true");
        }
        catch(IOException e) {
            if(e.getMessage().contains("404") || e.getMessage().contains("400")) {
                println "Request was successful but no environment with name ${envName} was found."
                apTool.setOutputProperty("exists", "false");
            }
            else {
                println "An error occurred during your request."
                throw new IOException(e);
            }
        }
        apTool.setOutputProperties()
    }
}
